package net.knarcraft.armoredelytra.listener;

import net.knarcraft.armoredelytra.ArmoredElytra;
import net.knarcraft.armoredelytra.config.message.Message;
import net.knarcraft.armoredelytra.event.ArmorEquipEvent;
import net.knarcraft.armoredelytra.metadata.DurabilityManager;
import net.knarcraft.armoredelytra.metadata.NBTEditor;
import net.knarcraft.armoredelytra.property.AllowedToWearEnum;
import net.knarcraft.armoredelytra.property.ArmorChangeMethod;
import net.knarcraft.armoredelytra.property.ArmorTier;
import net.knarcraft.armoredelytra.property.ArmorType;
import net.knarcraft.armoredelytra.util.GenericUtil;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.EntityDamageEvent.DamageCause;
import org.bukkit.event.player.PlayerItemMendEvent;
import org.bukkit.inventory.ItemStack;

import java.util.ArrayList;
import java.util.Random;

public class PlayerEventListener implements Listener {
    private final Random random = new Random();
    private final ArmoredElytra plugin;
    private final NBTEditor nbtEditor;
    private final DurabilityManager durabilityManager;

    public PlayerEventListener(ArmoredElytra plugin, NBTEditor nbtEditor, DurabilityManager durabilityManager) {
        this.plugin = plugin;
        this.nbtEditor = nbtEditor;
        this.durabilityManager = durabilityManager;
        initializeArmorEquipEvent();
    }

    private void initializeArmorEquipEvent() {
        Bukkit.getPluginManager().registerEvents(new ArmorListener(new ArrayList<>()), plugin);
        Bukkit.getPluginManager().registerEvents(new DispenserArmorListener(), plugin);
    }

    // Make sure the player has the correct permission and that the item is not
    // broken.
    private AllowedToWearEnum isAllowedToWear(ItemStack elytra, Player player, ArmorTier armorTier) {
        if (armorTier.equals(ArmorTier.NONE)) {
            return AllowedToWearEnum.ALLOWED;
        }
        if (GenericUtil.isBroken(elytra)) {
            return AllowedToWearEnum.BROKEN;
        }
        if (!plugin.playerHasWearPerm(player, armorTier)) {
            return AllowedToWearEnum.NO_PERMISSION;
        }
        return AllowedToWearEnum.ALLOWED;
    }

    // Handle armored elytra durability loss.
    @EventHandler(ignoreCancelled = true, priority = EventPriority.LOWEST)
    public void onPlayerDamage(EntityDamageEvent e) {
        if (!(e.getEntity() instanceof final Player p)) {
            return;
        }

        final ItemStack elytra = p.getInventory().getChestplate();
        if (elytra == null) {
            return;
        }

        final ArmorTier armorTier = nbtEditor.getArmorTier(elytra);
        if (armorTier == ArmorTier.NONE) {
            return;
        }

        final DamageCause cause = e.getCause();
        // The elytra doesn't receive any damage for these causes:
        if (cause == DamageCause.DROWNING || cause == DamageCause.STARVATION || cause == DamageCause.SUFFOCATION ||
                cause == DamageCause.SUICIDE || cause == DamageCause.FLY_INTO_WALL || cause == DamageCause.POISON) {
            return;
        }

        final boolean removeDurability;
        if (elytra.containsEnchantment(Enchantment.DURABILITY)) {
            final int randomInt = random.nextInt(101);
            final int enchantLevel = elytra.getEnchantmentLevel(Enchantment.DURABILITY);
            // Formula taken from: https://minecraft.fandom.com/wiki/Unbreaking#Usage
            final float removeDurabilityChance = 60 + 40f / (enchantLevel + 1);
            removeDurability = randomInt <= removeDurabilityChance;
        } else {
            removeDurability = true;
        }

        // Even when we don't subtract durability, we still want to update the durability, so just subtract 0.
        final int durabilityLoss = removeDurability ? (int) Math.max(1, e.getDamage() / 4) : 0;
        final int newDurability = durabilityManager.removeDurability(elytra, durabilityLoss, armorTier);
        if (durabilityManager.isBroken(newDurability, armorTier)) {
            GenericUtil.moveChestplateToInventory(p);
        }
    }

    @EventHandler(ignoreCancelled = true, priority = EventPriority.HIGHEST)
    public void onMending(PlayerItemMendEvent e) {
        final ArmorTier armorTier = nbtEditor.getArmorTier(e.getItem());
        if (armorTier == ArmorTier.NONE) {
            return;
        }
        final int newDurability = durabilityManager.removeDurability(e.getItem(), -e.getRepairAmount(), armorTier);

        // Apply it again a tick later, so we can override the durability of the armored elytra without
        // interfering with the player XP change event that depends on the success of this one.
        Bukkit.getScheduler().scheduleSyncDelayedTask(plugin, () ->
                durabilityManager.setDurability(e.getItem(), newDurability, armorTier), 1);
    }

    @EventHandler
    public void onEquip(ArmorEquipEvent equipEvent) {
        //If death caused it, or the item actually broke, do nothing
        if (equipEvent.getMethod().equals(ArmorChangeMethod.DEATH) ||
                equipEvent.getMethod().equals(ArmorChangeMethod.BROKEN)) {
            return;
        }

        //Check if an elytra is involved
        if (!equipEvent.getType().equals(ArmorType.CHESTPLATE) ||
                equipEvent.getNewArmorPiece() == null ||
                !equipEvent.getNewArmorPiece().getType().equals(Material.ELYTRA)) {
            return;
        }

        final ArmorTier armorTier = nbtEditor.getArmorTier(equipEvent.getNewArmorPiece());
        final AllowedToWearEnum allowed = isAllowedToWear(equipEvent.getNewArmorPiece(), equipEvent.getPlayer(),
                armorTier);
        switch (allowed) {
            case BROKEN -> {
                plugin.messagePlayer(equipEvent.getPlayer(), plugin.getMyMessages().getString(Message.MESSAGES_REPAIR_NEEDED));
                equipEvent.setCancelled(true);
            }
            case NO_PERMISSION -> {
                plugin.usageDeniedMessage(equipEvent.getPlayer(), armorTier);
                equipEvent.setCancelled(true);
            }
            default -> {
            }
        }
    }
}
