package net.knarcraft.armoredelytra.metadata;

import org.bukkit.enchantments.Enchantment;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

/**
 * An object for storing the original metadata for an armored elytra.
 *
 * <p>The intended usage of this class is to keep track of the original metadata of the armor and the elytra for each
 * armored elytra, to prevent problems when splitting them. If an armor used to upgrade the armored elytra has a custom
 * name or lore, and none is stored already, the name and lore should be updated. Additionally, any enchantments found
 * on armor used to upgrade the armored elytra should be combined with the original armor enchantments.</p>
 */
public class OriginalMetadata implements Serializable {

    private Set<Enchantment> armorEnchantments = new HashSet<>();
    private Set<Enchantment> elytraEnchantments = new HashSet<>();
    private String elytraLore;
    private String elytraName;
    private String armorLore;
    private String armorName;

    public OriginalMetadata() {

    }

    /**
     * Gets the armor enchantments stored in this metadata.
     *
     * @return <p>The stored armor enchantments.</p>
     */
    public @NotNull Set<Enchantment> getArmorEnchantments() {
        return new HashSet<>(armorEnchantments);
    }

    /**
     * Gets the elytra enchantments stored in this metadata.
     *
     * @return <p>The stored elytra enchantments.</p>
     */
    public @NotNull Set<Enchantment> getElytraEnchantments() {
        return new HashSet<>(elytraEnchantments);
    }

    /**
     * Gets the original lore of this elytra.
     *
     * @return <p>The original elytra lore.</p>
     */
    public @Nullable String getElytraLore() {
        return this.elytraLore;
    }

    /**
     * Gets the original name of this elytra.
     *
     * @return <p>The original elytra name.</p>
     */
    public @Nullable String getElytraName() {
        return this.elytraName;
    }

    /**
     * Gets the original lore of this armor.
     *
     * @return <p>The original armor lore.</p>
     */
    public @Nullable String getArmorLore() {
        return this.armorLore;
    }

    /**
     * Gets the original name of this armor.
     *
     * @return <p>The original armor name.</p>
     */
    public @Nullable String getArmorName() {
        return this.armorName;
    }

    /**
     * Sets the stored lore of this armored elytra's armor.
     *
     * @param armorLore <p>The armor lore to store.</p>
     * @return <p>This original metadata.</p>
     */
    public OriginalMetadata setArmorLore(@NotNull String armorLore) {
        this.armorLore = armorLore;
        return this;
    }

    /**
     * Sets the stored elytra lore of this armored elytra's elytra.
     *
     * @param elytraLore <p>The elytra lore to store.</p>
     * @return <p>This original metadata.</p>
     */
    public OriginalMetadata setElytraLore(@NotNull String elytraLore) {
        this.elytraLore = elytraLore;
        return this;
    }

    /**
     * Sets the stored elytra name of this armored elytra's elytra.
     *
     * @param elytraName <p>The elytra name to store.</p>
     * @return <p>This original metadata.</p>
     */
    public OriginalMetadata setElytraName(@NotNull String elytraName) {
        this.elytraName = elytraName;
        return this;
    }

    /**
     * Sets the stored armor name of this armored elytra's armor.
     *
     * @param armorName <p>The armor name to store.</p>
     * @return <p>This original metadata.</p>
     */
    public OriginalMetadata setArmorName(@NotNull String armorName) {
        this.armorName = armorName;
        return this;
    }

    /**
     * Sets the stored enchantments for this armored elytra's armor.
     *
     * @param armorEnchantments <p>The armor enchantments to store.</p>
     * @return <p>This original metadata.</p>
     */
    public OriginalMetadata setArmorEnchantments(@NotNull Set<Enchantment> armorEnchantments) {
        this.armorEnchantments = armorEnchantments;
        return this;
    }

    /**
     * Sets the stored enchantments for this armored elytra's elytra.
     *
     * @param elytraEnchantments <p>The elytra enchantments to store.</p>
     * @return <p>This original metadata.</p>
     */
    public OriginalMetadata setElytraEnchantments(@NotNull Set<Enchantment> elytraEnchantments) {
        this.elytraEnchantments = elytraEnchantments;
        return this;
    }

    /**
     * Adds the given enchantments to armor enchantments.
     *
     * <p>The purpose of this method is to combine enchantments for the original chestplate, and any chest-plates used
     * to upgrade the armored elytra.</p>
     *
     * @param enchantments <p>The enchantments to add</p>
     * @return <p>This original metadata.</p>
     */
    public OriginalMetadata addArmorEnchantments(@NotNull Set<Enchantment> enchantments) {
        this.armorEnchantments.addAll(enchantments);
        return this;
    }

}
