package net.knarcraft.armoredelytra.update;

import com.google.common.base.Preconditions;

/**
 * Represents a result for an update query performed by {@link UpdateChecker#requestUpdateCheck()}.
 */
public final class UpdateResult {

    private final UpdateReason reason;
    private final String newestVersion;
    private final long age;

    UpdateResult(UpdateChecker updateChecker, final UpdateReason reason, final String newestVersion, final long age) {
        updateChecker.lastResult = this;
        this.reason = reason;
        this.newestVersion = newestVersion;
        this.age = age;
    }

    UpdateResult(UpdateChecker updateChecker, final UpdateReason reason) {
        Preconditions
                .checkArgument(reason != UpdateReason.NEW_UPDATE && reason != UpdateReason.UP_TO_DATE,
                        "Reasons that might require updates must also provide the latest version String");
        this.reason = reason;
        newestVersion = updateChecker.plugin.getDescription().getVersion();
        age = -1;
    }

    /**
     * Gets the constant reason of this result.
     *
     * @return the reason
     */
    public UpdateReason getReason() {
        return reason;
    }

    /**
     * Checks whether this result requires the user to update.
     *
     * @return true if an update is required, false otherwise
     */
    public boolean requiresUpdate() {
        return reason == UpdateReason.NEW_UPDATE;
    }

    /**
     * Gets the latest version of the plugin. This may be the currently installed version, it may not be. This
     * depends entirely on the result of the update.
     *
     * @return the newest version of the plugin
     */
    public String getNewestVersion() {
        return newestVersion;
    }

    /**
     * Gets the number of seconds since the last update was released.
     *
     * @return The number of seconds since the last update was released or -1 if unavailable.
     */
    public long getAge() {
        return age;
    }

}
